use Modern::Perl;
package Intertangle::Yarn;
# ABSTRACT: Geometry primitives for graphics
$Intertangle::Yarn::VERSION = '0.002';
use Intertangle::Yarn::Graphene;

1;

__END__

=pod

=encoding UTF-8

=head1 NAME

Intertangle::Yarn - Geometry primitives for graphics

=head1 VERSION

version 0.002

=head1 SEE ALSO

=head1 AUTHOR

Zakariyya Mughal <zmughal@cpan.org>

=head1 COPYRIGHT AND LICENSE

This software is copyright (c) 2017 by Zakariyya Mughal.

This is free software; you can redistribute it and/or modify it under
the same terms as the Perl 5 programming language system itself.

=cut
